{
  self,
  nixpkgs,
  inputs,
}:

let
  mkSystem =
    systemConfig: name:
    nixpkgs.lib.nixosSystem rec {
      system =
        builtins.replaceStrings
          [
            "-unknown-"
            "-gnu"
          ]
          [
            "-"
            ""
          ]
          systemConfig;
      specialArgs = inputs;
      modules = [
        ./systems/${name}
        self.outputs.nixosModules.default
        {
          system.stateVersion = "24.05";
          nixpkgs.hostPlatform = {
            system = system;
            config = systemConfig;
          };
          networking.hostName = name;

          nix = {
            settings = {
              substituters = lib.mkForce [
                "https://cache.nixos.org"
                "https://nix-community.cachix.org"
              ];
              trusted-public-keys = lib.mkForce [
                "cache.nixos.org-1:6NCHdD59X431o0gWypbMrAURkbJ16ZPMQFGspcDShjY="
                "nix-community.cachix.org-1:mB9FSh9qf2dCimDSUo8Zy7bkq5CX+/rkCWyvRCYg3Fs="
              ];
              trusted-users = [ "root" ];
              sandbox = true;
              require-sigs = true;
              max-jobs = "auto";
              auto-optimise-store = true;
              allowed-users = [ "*" ];
              experimental-features = [
                "nix-command"
                "flakes"
              ];
              warn-dirty = true;
              use-xdg-base-directories = true;
              keep-going = true;
              builders-use-substitutes = true;
            };
            distributedBuilds = true;
            package = pkgs.nixVersions.unstable;
            registry = lib.mkForce (
              (lib.mapAttrs (_: flake: { inherit flake; })) (
                (lib.filterAttrs (_: lib.isType "flake")) self.inputs
              )
              // {
                self.flake = self;
              }
            );
            nixPath = lib.mkForce [ "/etc/nix/path" ];
          };

          nixpkgs = {
            config = {
              allowUnfree = true;
              nvidia.acceptLicense = true;
            };
          };

          environment.etc = lib.mapAttrs' (name: value: {
            name = "nix/path/${name}";
            value.source = value.flake;
          }) config.nix.registry;

          home-manager = {
            useGlobalPkgs = true;
            useUserPackages = true;
            backupFileExtension = "backup";
          };

          systemd.services.nix-daemon.serviceConfig.LimitNOFILE = lib.mkForce 1048576000;

          networking.hosts = {
            "192.168.191.46" = [ "zap" ];
            "192.168.191.142" = [ "kuraokami" ];
            "2a01:4f8:1c1e:7f93::1" = [ "bloomshine.de" ];
          };

          hardware.bluetooth.enable = true;
          hardware.bluetooth.powerOnBoot = true;

          services.openssh = {
            enable = true;
            banner = "Hackers are in Your System!!!\n";
            settings = {
              PasswordAuthentication = false;
              PermitRootLogin = "prohibit-password";
            };
          };
        }
      ];
    };
in
{
  sukuna-biko-na = mkSystem "aarch64-unknown-linux-gnu" "sukuna-biko-na";
  raspberry4 = mkSystem "aarch64-unknown-linux-gnu" "raspberry4";
}
